#!/usr/bin/perl
# Filename:      mailarchiv.pl
# Version:       0.5
#
# Copyright (C) 2004, Megabit Informationstechnik GmbH <info@megabit.net>
#
# Description:   reads internet mail message file from stdin, analyses it and stores
#                basic information into a database. Afterwards the message file is
#                stored in a directory structure which can be easily backuped by
#                external software
# Author:        Daniel Weuthen <dw@megabit.net>
# Created at:    Thu Sep 02 13:48:49 2004
#
# This program is free software; you can redistribute
# it and/or modify it under the terms of the GNU General
# Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place,
# Suite 330, Boston, MA  02111-1307  USA

require('/etc/mailarchiv.conf');

use Mail::Header;
use Mail::Internet;
use FileHandle;
use DBI;
use HTTP::Date;
use MIME::Words qw(:all);
use Encode;
use Data::Uniqid qw ( luniqid );
use Digest::MD5;
use File::Path;
use File::Copy;
use Sys::Syslog;

my $md5=new Digest::MD5;

my $header_var;
my %m_header_vars = ("guid","",
                     "md5sum","",
                     "date","",
                     "message_id","",
                     "from","",
                     "sender","",
                     "subject","",
                     "to","",
                     "path","");


my $header_end_reached = 0;
my $tmpfile = "/tmp/archiv$$.tmp";

open (TMPFILE,"> $tmpfile") or &my_error("ERROR: $tmpfile".$!);
while (<>){
  $zeile = $_;
  $zeile_nsp = $zeile;
  print TMPFILE $zeile;
  $zeile_nsp =~ s/ //g;
  chomp($zeile_nsp);
  if ( length($zeile_nsp) == 0 ) { $header_end_reached = 1 }
  if ( $header_end_reached == 0 ) {
    @message = (@message, $zeile);
  }
}
close (TMPFILE);

$header = new Mail::Header \@message;

$m_header_vars{"guid"}        = luniqid;
$m_header_vars{"md5sum"}      = &my_md5sum;

if ( &my_mysql_datetime($header->get("Date")) =~ /^([0-9\S\W]{4,4})-([0-9\S\W]{2,2})-([0-9\S\W]{2,2}) ([0-9\S\W]{2,2}):([0-9\S\W]{2,2}):([0-9\S\W]{2,2})/ ) {
  $m_header_vars{"date"}      = &my_mysql_datetime($header->get("Date"));
} else {
  ($Second, $Minute, $Hour, $Day, $Month, $Year, $WeekDay, $DayOfYear, $IsDST) = localtime(time);
  $Month++;
  $Year += 1900;
  if ( $Second < 10 ) { $Second = "0$Second" }
  if ( $Minute < 10 ) { $Minute = "0$Minute" }
  if ( $Hour < 10 ) { $Hour = "0$Hour" }
  if ( $Day < 10 ) { $Day = "0$Day" }
  if ( $Month < 10 ) { $Month = "0$Month" }
  $m_header_vars{"date"}      = "$Year-$Month-$Day $Hour:$Minute:$Second";
}

$m_header_vars{"message_id"}  = $header->get("Message-Id");
$m_header_vars{"from"}        = decode_mimewords($header->get("From"));
if ( length($m_header_vars{"from"}) == 0 ) { $m_header_vars{"from"} = decode_mimewords($header->get("Sender")); }
if ( length($m_header_vars{"from"}) == 0 ) { $m_header_vars{"from"} = decode_mimewords($header->get("Reply-To")); }
$m_header_vars{"sender"}      = decode_mimewords($header->get("Sender"));
$m_header_vars{"replyto"}     = decode_mimewords($header->get("Reply-To"));
$m_header_vars{"subject"}     = decode_mimewords($header->get("Subject"));
$m_header_vars{"to"}          = decode_mimewords($header->get("To"));

# parse mysql formated date and create complete path to message in archive
$date = $m_header_vars{"date"};
$date =~ /^([0-9\S\W]{4,4})-([0-9\S\W]{2,2})-([0-9\S\W]{2,2}) ([0-9\S\W]{2,2}):([0-9\S\W]{2,2}):([0-9\S\W]{2,2})/;
$year  = $1;
$month = $2;
$day   = $3;

$directory = "$base/$year/$month/$day";
$filename = $m_header_vars{"guid"}."\.msg";

# create directory if not exists and copy temporary file into archiv
unless (-d "$directory") { mkpath($directory) or &my_error("ERROR: could not create archiv directory") };
copy($tmpfile,"$directory/$filename") or &my_error("ERROR: could not copy temporaray file to archiv");
$m_header_vars{"path"} = "$directory/$filename";

# connect to database, must be established here because usage of quote command from $dbh construct in foreach loop
$dbh = DBI->connect("DBI:mysql:$db_database:$db_host:$db_sp", "$db_user", "$db_pass") or &my_error("ERROR: ".$DBI::errstr);

# encode header variables to utf8 and substitute empty header with string "<Leer>"
foreach $header_var (values %m_header_vars)
{
  chomp($header_var);
  $header_var = encode("utf8", $header_var);
  if (length($header_var) == 0) { $header_var = "<Leer>" };
  $header_var = $dbh->quote($header_var);
}

# write header information into database
my $SQL = "INSERT INTO archiv VALUES (".$m_header_vars{"guid"}.",".$m_header_vars{"md5sum"}.",".$m_header_vars{"date"}.",".$m_header_vars{"from"}.",".$m_header_vars{"to"}.",".$m_header_vars{"subject"}.",".$m_header_vars{"message_id"}.",".$m_header_vars{"path"}.")";
my $cursor = $dbh->prepare($SQL) or &my_error("ERROR: ".$dbh->errstr);
$cursor->execute or &my_error("ERROR: ".$dbh->errstr);
$dbh->disconnect;

# print debugging information
if ($debug == 1) {
  foreach $line (@message) {
    print $line;
  }
  print "GUID .........: ",$m_header_vars{"guid"},"\n";
  print "MD5 Summe ....: ",$m_header_vars{"md5sum"},"\n";
  print "Date .........: ",$m_header_vars{"date"},"\n";
  print "Message-Id ...: ",$m_header_vars{"message_id"},"\n";
  print "From .........: ",$m_header_vars{"from"},"\n";
  print "Sender .......: ",$m_header_vars{"sender"},"\n";
  print "Reply-To .....: ",$m_header_vars{"replyto"},"\n";
  print "Subject ......: ",$m_header_vars{"subject"},"\n";
  print "To ...........: ",$m_header_vars{"to"},"\n";
  print "Path .........: ",$m_header_vars{"path"},"\n";
}

# exit without error
&my_exit("SUCCESS: archiving mail from ".$m_header_vars{"from"}." to $directory/$filename (md5sum=".$m_header_vars{"md5sum"}.")");

# convert date string from header to mysql date format
sub my_mysql_datetime($dt)
{
  my $dt = shift;
  ($Year, $Month, $Day, $Hour, $Minute, $second, $tz) = HTTP::Date::parse_date($dt);
  if ( $Second < 10 ) { $Second = "0$Second" }
  if ( $Minute < 10 ) { $Minute = "0$Minute" }
  if ( $Hour < 10 ) { $Hour = "0$Hour" }
  if ( $Day < 10 ) { $Day = "0$Day" }
  if ( $Month < 10 ) { $Month = "0$Month" }
  return "$Year-$Month-$Day $Hour:$Minute:$Second";
}

# return md5sum of curent messagefile
sub my_md5sum
{
  my $message=shift;
  if (my $filehandle=new FileHandle "< $tmpfile")
  {
    $md5->reset();
    $md5->addfile($filehandle);
    return $md5->hexdigest();
  } else {
    &my_error("ERROR: could not calculate md5sum");
  }
}

# write to logfile
sub my_do_log
{ 
  my $log_msg = shift;
  openlog('mailarchiv', 'cons,pid', 'user');
  syslog('info', $log_msg);
  closelog();
}

# error handling
sub my_error
{
  my $error_msg = shift;
  &my_do_log($error_msg);
  unlink($tmpfile);
  unlink("$directoryi/$filename");
  exit 1
}

# normal termination
sub my_exit
{
  my $exit_msg = shift;
  &my_do_log($exit_msg);
  unlink($tmpfile);
  exit 0
}
